@@ -32,7 +32,7 @@
 
 	if [ -e /etc/nsswitch.conf ]; then
 		INST_CFG /etc/nsswitch.conf
-		databases="$(awk '/^group|hosts/ {for(i = 2; i <= NF; ++i) print $i}' /etc/nsswitch.conf | sort | uniq)"
+		databases="$(awk '/^group|hosts/ {for(i = 2; i <= NF; ++i) print $i}' /etc/nsswitch.conf | LC_ALL=C sort -u)"
 		for db in $databases; do
 			for f in /lib/*/"libnss_$db"*; do
 				INST_LIB "$f"
