@@ -2,13 +2,13 @@
 .
 .Sh SPECIAL THANKS
 To all who support further development, in particular:
-.Bl -bullet -offset 4n -compact -width 0
+.Bl -bullet -offset 4n -compact -width "@"
 .It
 ThePhD
 .It
 Embark Studios
 .It
-Jasper Bekkers
+Lars Strojny
 .El
 .
 .Sh REPORTING BUGS
