@@ -1,7 +1,7 @@
 .\" SPDX-License-Identifier: MIT
 .
 .Sh ENVIRONMENT VARIABLES
-.Bl -tag -compact -width "TZPFMS"
+.Bl -tag -compact -width 4n
 .It Ev TZPFMS_PASSPHRASE_HELPER
 By default, passphrases are prompted for and read in on the standard output and input streams.
 If
@@ -12,7 +12,7 @@ to provide each passphrase, instead.
 .Pp
 The standard output stream of the helper is tied to an anonymous file and used in its entirety as the passphrase, except for a trailing new-line, if any.
 The arguments are:
-.Bl -tag -compact -offset "@@" -width "@@"
+.Bl -tag -compact -offset 2n -width ".Li $1"
 .It Li $1
 Pre-formatted noun phrase with all the information below, for use as a prompt
 .\" Passphrase for tarta-zoot
