@@ -12,7 +12,6 @@
 #define OPENSSL_SUPPRESS_DEPRECATED  // SHA256_*(); supposedly replaced with EVP* but that's horseshit. we'll see how she turns out, given there's no reason given
 #include <openssl/sha.h>
 #include <optional>
-#include <time.h>
 #include <utility>
 
 
@@ -43,21 +42,6 @@ static int try_or_passphrase(const char * what, const char * what_for, ESYS_CONT
 }
 
 
-TPM2B_DATA tpm2_creation_metadata(const char * dataset_name) {
-	TPM2B_DATA metadata{};  // 64 bytesish
-
-	struct timespec ts;
-	clock_gettime(CLOCK_REALTIME, &ts);
-	metadata.size = snprintf((char *)metadata.buffer, sizeof(metadata.buffer), "%" PRId64 ".%09" PRId64 " %s %s", static_cast<int64_t>(ts.tv_sec),
-	                         static_cast<int64_t>(ts.tv_nsec), dataset_name, TZPFMS_VERSION) +
-	                1;
-	metadata.size = metadata.size > sizeof(metadata.buffer) ? sizeof(metadata.buffer) : metadata.size;
-
-	// fprintf(stderr, "%" PRIu16 "/%zu: \"%s\"\n", metadata.size, sizeof(metadata.buffer), metadata.buffer);
-	return metadata;
-}
-
-
 int tpm2_parse_prop(const char * dataset_name, char * handle_s, TPMI_DH_PERSISTENT & handle, TPML_PCR_SELECTION * pcrs) {
 	char * sv{};
 	if(!parse_uint(handle_s = strtok_r(handle_s, ";", &sv), handle))
@@ -315,18 +299,19 @@ static int tpm2_police_pcrs(ESYS_CONTEXT * tpm2_ctx, const TPML_PCR_SELECTION &
 	return with_session(pcr_session);
 }
 
-int tpm2_seal(const char * dataset, ESYS_CONTEXT * tpm2_ctx, ESYS_TR tpm2_session, TPMI_DH_PERSISTENT & persistent_handle, const TPM2B_DATA & metadata,
+int tpm2_seal(const char * dataset, ESYS_CONTEXT * tpm2_ctx, ESYS_TR tpm2_session, TPMI_DH_PERSISTENT & persistent_handle,
               const TPML_PCR_SELECTION & pcrs, bool allow_PCR_or_pass, void * data, size_t data_len) {
 	ESYS_TR primary_handle = ESYS_TR_NONE;
 	quickscope_wrapper primary_handle_deleter{[&] { Esys_FlushContext(tpm2_ctx, primary_handle); }};
 
+	const TPM2B_DATA metadata{};
 	{
 		const TPM2B_SENSITIVE_CREATE primary_sens{};
 
 		// Adapted from tpm2-tss-3.0.1/test/integration/esys-create-primary-hmac.int.c
 		TPM2B_PUBLIC pub{};
 		pub.publicArea.type             = TPM2_ALG_RSA;
-		pub.publicArea.nameAlg          = TPM2_ALG_SHA1;
+		pub.publicArea.nameAlg          = TPM2_ALG_SHA256;
 		pub.publicArea.objectAttributes = TPMA_OBJECT_USERWITHAUTH | TPMA_OBJECT_RESTRICTED | TPMA_OBJECT_DECRYPT | TPMA_OBJECT_FIXEDTPM | TPMA_OBJECT_FIXEDPARENT |
 		                                  TPMA_OBJECT_SENSITIVEDATAORIGIN;
 		pub.publicArea.parameters.rsaDetail.symmetric.algorithm   = TPM2_ALG_AES;
@@ -335,6 +320,10 @@ int tpm2_seal(const char * dataset, ESYS_CONTEXT * tpm2_ctx, ESYS_TR tpm2_sessio
 		pub.publicArea.parameters.rsaDetail.scheme.scheme         = TPM2_ALG_NULL;
 		pub.publicArea.parameters.rsaDetail.keyBits               = 2048;
 		pub.publicArea.parameters.rsaDetail.exponent              = 0;
+		pub.publicArea.parameters.asymDetail.scheme.scheme        = TPM2_ALG_NULL;
+		pub.publicArea.parameters.symDetail.sym.algorithm         = TPM2_ALG_AES;
+		pub.publicArea.parameters.symDetail.sym.keyBits.sym       = 128;
+		pub.publicArea.parameters.symDetail.sym.mode.sym          = TPM2_ALG_CFB;
 		TRY_MAIN(try_or_passphrase("create primary encryption key", "TPM2 owner hierarchy", tpm2_ctx, TPM2_RC_BAD_AUTH, ESYS_TR_RH_OWNER, [&] {
 			return Esys_CreatePrimary(tpm2_ctx, ESYS_TR_RH_OWNER, tpm2_session, ESYS_TR_NONE, ESYS_TR_NONE, &primary_sens, &pub, &metadata, &pcrs, &primary_handle,
 			                          nullptr, nullptr, nullptr, nullptr);
